// $Id: coralServer.cpp,v 1.12 2009/07/20 13:44:08 avalassi Exp $

// Include files
#include <iostream>
#include <stdlib.h> // For setenv and _putenv
#include "CoralKernel/Context.h"
#include "CoralSockets/SocketServer.h"
#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#define LOGGER_NAME CoralServer::logger // dummy (avoid compilation #error)
#include "CoralServerBase/logger.h"

// Local include files
#include "CoralServerFacadeFactory.h"
#include "ServerStubFactory.h"
#include "ThreadMonitorReporter.h"

// Message output
#define LOG std::cout << "__coralServer "

// Namespace
using namespace coral;

//-----------------------------------------------------------------------------

/** @file coralServer.cpp
 *
 *  @author Andrea Valassi
 *  @date   2009-02-09
 */

int main( int argc, char** argv )
{

  LOG << "Entering main" << std::endl;
  try 
  {
    
    // Get the server parameters
    std::string host;
    int port;
    int timeout;
    int monitorPeriod;
    std::string replicaSvc;
    if ( argc == 1 ) 
    {
      host = "0.0.0.0";
      port = 40007;
      timeout = 30; // -1 means the server never stops
      monitorPeriod = 0;
      replicaSvc = "";
    }
    else if ( argc == 4 || argc == 5 || argc == 6 ) 
    {
      host = argv[1];
      port = atoi( argv[2] );
      timeout = atoi( argv[3] );
      if ( argc <= 4 ) monitorPeriod = 0;
      else monitorPeriod = atoi( argv[4] );
      if ( argc <= 5 ) replicaSvc = "";
      else replicaSvc = argv[5];
    } 
    else 
    {
      LOG << "Usage:   " << argv[0] << " [host port timeout [monitorPeriod [replicaSvc]]]" << std::endl;
      LOG << "Example: " << argv[0] << " 0.0.0.0 40007 30" << std::endl;
      LOG << "Example: " << argv[0] << " 0.0.0.0 40007 30 10" << std::endl;
      LOG << "Example: " << argv[0] << " 0.0.0.0 40007 30 10 CORAL/Services/CoralAuthenticationService" << std::endl;
      return 1;
    }

    // Setup CORAL db lookup and authentication from the user's AFS private 
    /*
    std::string user( getenv("USER") );
    std::string afsPrivate = "/afs/cern.ch/user/"+user.substr(0,1)+"/"+user+"/private/CoralServer";
#ifdef WIN32
    _putenv( ( "CORAL_DBLOOKUP_PATH=" + afsPrivate ).c_str() );
    _putenv( ( "CORAL_AUTH_PATH" + afsPrivate ).c_str() );
#else
    setenv( "CORAL_DBLOOKUP_PATH", afsPrivate.c_str(), 1 );
    setenv( "CORAL_AUTH_PATH", afsPrivate.c_str(), 1 );
#endif
    */
    if ( !getenv( "CORAL_AUTH_PATH" ) )
    {
      LOG << "ERROR! Env CORAL_AUTH_PATH is not defined" << std::endl;
      return 1;
    }
    if ( !getenv( "CORAL_DBLOOKUP_PATH" ) )
    {
      LOG << "ERROR! Env CORAL_DBLOOKUP_PATH is not defined" << std::endl;
      return 1;
    }
    LOG << "Using CORAL_AUTH_PATH='" 
        << getenv( "CORAL_AUTH_PATH" ) <<  "'" << std::endl;
    LOG << "Using CORAL_DBLOOKUP_PATH='" 
        << getenv( "CORAL_DBLOOKUP_PATH" ) <<  "'" << std::endl;

    // Load ConnectionService2
    std::string connSvc2Name = "CORAL/Services/ConnectionService2";
    LOG << "Load " << connSvc2Name << std::endl;
    Context::instance().loadComponent( connSvc2Name );
    IHandle<IConnectionService> connSvc2 = 
      Context::instance().query<IConnectionService>( connSvc2Name );
    if ( !connSvc2.isValid() )
    {
      LOG << "ERROR! Could not load " << connSvc2Name << std::endl;
      return 1;
    }

    // Configure ConnectionService2
    LOG << "Configure " << connSvc2Name << std::endl;
    connSvc2->configuration().disablePoolAutomaticCleanUp(); 
    connSvc2->configuration().setConnectionTimeOut( 0 );
    if ( replicaSvc != "" )
    {
      connSvc2->configuration().setAuthenticationService( replicaSvc );
      connSvc2->configuration().setLookupService( replicaSvc );
    }    

    // Start the coralServer
    {
      LOG << "Create a CoralServerFacadeFactory" << std::endl;
      CoralServer::CoralServerFacadeFactory facadeFactory;
      LOG << "Create a ServerStubFactory" << std::endl;
      CoralServer::ServerStubFactory handlerFactory( facadeFactory );
      LOG << "Create the server on " << host << ":" << port << std::endl;
      CoralSockets::SocketServer server( handlerFactory, host, port );
      {
        LOG << "Create the ThreadMonitorReporter" << std::endl;
        CoralServer::ThreadMonitorReporter 
          reporter( *(server.threadMonitor()), monitorPeriod );
        LOG << "Start the server (timeout = " << timeout << "s)" << std::endl;
        server.run( timeout );
        LOG << "The server has been stopped" << std::endl;
        LOG << "Delete the ThreadMonitorReporter" << std::endl;
      }
      LOG << "Delete the server" << std::endl;
    }
  }
  
  catch( std::exception& e )
  {
    LOG << "ERROR! Standard C++ exception: '" << e.what() << "'" << std::endl;
    return 1;
  }

  catch( ... ) 
  {
    LOG << "ERROR! Unknown exception caught" << std::endl;
    return 1;
  }

  // Successful program termination
  LOG << "Exiting main" << std::endl;
  return 0;
  
}

//-----------------------------------------------------------------------------
