// $Id: test_ThreadManager.cpp,v 1.4 2009/07/15 07:19:41 avalassi Exp $

#include <iostream>

#include "../../../CoralServerBase/tests/Common/CoralServerUnitTest.h"

// Local include files 
#include "../../src/ThreadManager.h"

// Namespace
using namespace coral::CoralSockets;
using namespace coral;

namespace coral 
{

  class ThreadTest : public Runnable {
    public:
       ThreadTest( boost::mutex &mutex, int &count )
         : m_mutex( mutex )
           , m_count( count )
    {};
   
    const std::string desc() const
    {
      return "ThreadTest";
    }     
     
       void endThread()
       {}

       void operator()() {
         boost::mutex::scoped_lock lock( m_mutex );
         m_count++;
       }
    private:
       boost::mutex & m_mutex;
       int & m_count;
  };

  class ThreadManagerTest : public CoralServerUnitTest 
  {

    CPPUNIT_TEST_SUITE( ThreadManagerTest );
    CPPUNIT_TEST( test_Basic );
    CPPUNIT_TEST( test_100Threads );
    CPPUNIT_TEST( test_1000Threads );
    CPPUNIT_TEST_SUITE_END();

    public:

    void setUp() {}

    void tearDown() {
    }


    // ------------------------------------------------------

    void test_Basic() 
    {
      ThreadManager manager;
      boost::mutex mutex;
      int count=0;

      manager.addThread( new ThreadTest( mutex, count ) );
      manager.joinAll();
      CPPUNIT_ASSERT_EQUAL_MESSAGE("count 1", 1, count );

      // can't add one class twice
      ThreadTest *test=new ThreadTest( mutex, count );
      manager.addThread( test );
      CPPUNIT_ASSERT_THROW( manager.addThread( test ),
          GenericSocketException );

      // make sure the threads are properly cleaned up
      CPPUNIT_ASSERT_EQUAL_MESSAGE("existsThread 1",
          true, manager.existsThread( test ) );
      manager.joinAll();
      manager.housekeeping();
      CPPUNIT_ASSERT_EQUAL_MESSAGE("existsThread 2",
          false, manager.existsThread( test ) );

    }

    // ------------------------------------------------------

    void test_100Threads() 
    {
      ThreadManager manager;
      boost::mutex mutex;
      int count=0;

      for ( int i=0; i < 100; i++ ) {
        manager.addThread( new ThreadTest(mutex, count) );
      }
      manager.joinAll();
      // make sure every thread has finished
      CPPUNIT_ASSERT_EQUAL_MESSAGE("count 1", 100, count );

    }

    // ------------------------------------------------------

    void test_1000Threads() 
    {
      ThreadManager manager;
      boost::mutex mutex;
      int count=0;

      for ( int i=0; i < 1000; i++ ) {
        manager.addThread( new ThreadTest(mutex, count) );
      }
      manager.joinAll();
      // make sure every thread has finished
      CPPUNIT_ASSERT_EQUAL_MESSAGE("count 1", 1000, count );

    }
  };

  CPPUNIT_TEST_SUITE_REGISTRATION( ThreadManagerTest );
  
}

CORALSERVERTEST_MAIN( ThreadManagerTest )
