#! /usr/local/bin/bash

if [ "$1" != "local" ] && [ "$1" != "server" ] && [ "$1" != "proxy" ]; then
  echo "Usage: $0 (local|server|proxy) (stdout|file)"
  exit 1
else
  conn=$1
fi

if [ "$2" == "stdout" ]; then
  out=/dev/stdout
  echo "Output on stdout ($out)"
elif [ "$2" == "file" ]; then
  out=logs/athenaMT_${conn}.txt
  echo "Output on file ($out)"
  \rm -rf $out
else
  echo "Usage: $0 (local|server|proxy) (stdout|file)"
  exit 1
fi

vmemSet=1536000
ulimit -v $vmemSet >> $out 2>&1 
vmem=`ulimit -v`
if [ $vmem != $vmemSet ]; then
  echo "ERROR! Insufficient vmem $vmem (expected $vmemSet)" > /dev/stderr
  echo "ERROR! Insufficient vmem $vmem (expected $vmemSet)" >> $out
  exit 1
fi

cmdDir=`dirname $0`
cmdDir=`cd $cmdDir; pwd`
pushd $cmdDir > /dev/null
. setupAthenaMT.sh $conn >> $out 2>&1 
if [ "$?" != "0" ]; then exit 1; fi
popd > /dev/null

echo "*** Execute athenaMT" >> $out
echo CMTCONFIG=$CMTCONFIG >> $out
echo HOST=$HOST >> $out
echo VirtualMemory=$vmem >> $out

# Use one or more dbg (rather than opt) libraries
###\rm -rf $cmdDir/tmp
###mkdir $cmdDir/tmp
###ln -sf /afs/cern.ch/atlas/software/builds/AtlasCore/15.2.0/InstallArea/i686-slc4-gcc34-dbg/lib/libIOVDbSvc.so $cmdDir/tmp
###export LD_LIBRARY_PATH=$cmdDir/tmp:$LD_LIBRARY_PATH
######export LD_LIBRARY_PATH=/afs/cern.ch/atlas/software/builds/AtlasCore/15.2.0/InstallArea/i686-slc4-gcc34-dbg/lib:$LD_LIBRARY_PATH

# Use a private build of the IOVDbSvc library
###export LD_LIBRARY_PATH=$cmdDir/myAthenaSrc/Database/IOVDbSvc/i686-slc4-gcc34-dbg:$LD_LIBRARY_PATH

# Define the debug level
athenaDebugLevel=
###athenaDebugLevel='--log-level=DEBUG'

# Define the number of threads
athenaNThreads=
###athenaNThreads='--number-of-workers=1' # WARNING! Segfault in TFile::Write()

unset CORAL_MSGLEVEL
export CORAL_MESSAGEREPORTER_FORMATTED=1

###echo LD_LIBRARY_PATH=$LD_LIBRARY_PATH >> $out
######echo $LD_LIBRARY_PATH | tr : "\n"  >> $out
######ldd /afs/cern.ch/user/a/avalassi/myLCG/CORAL_HEAD/slc4_ia32_gcc34_dbg/lib/liblcg_CoralAccess.so >> $out
date >> $out
/usr/bin/time -ao/dev/stdout -f"=== TIMER (CLIENT) %Uuser %Ssystem %eelapsed %PCPU" athenaMT.py $athenaDebugLevel $athenaNThreads -f daq.lumi1E31.latest.data -c "test1E31MC=True" runHLT_standalone.py >> $out 2>&1 
###/usr/bin/time -ao/dev/stdout -f"=== TIMER (CLIENT) %Uuser %Ssystem %eelapsed %PCPU" athenaMT.py -d gdb -f daq.lumi1E31.latest.data -c "test1E31MC=True" runHLT_standalone_gdb.py >> $out 2>&1 
date >> $out

if [ "$out" != "/dev/stdout" ]; then
  \mv ${out} ${out}.old
  $cmdDir/sedDate.csh ${out}.old > ${out}
fi
