#ifndef CORAL_CORALBASE_MSGREPORTER2_H
#define CORAL_CORALBASE_MSGREPORTER2_H 1

#include <boost/thread/mutex.hpp>
#include "CoralBase/MessageStream.h"

namespace coral
{

  namespace CoralBase
  {

    /** @class MsgReporter2 MsgReporter2.h
     *
     * Thread-safe reporter implementation
     *
     * @author   Markus Frank, Martin Wache, Andrea Valassi
     */

    class MsgReporter2 : public IMsgReporter
    {

    public:

      /// Default constructor
      MsgReporter2();

      /// Destructor
      virtual ~MsgReporter2() {}

      /// Release reference to reporter
      virtual void release() 
      {
        delete this;        
      }

      /// Access output level
      virtual coral::MsgLevel outputLevel() const;

      /// Modify output level
      virtual void setOutputLevel( coral::MsgLevel lvl );

      /// Report a message
      virtual void report( int lvl, 
                           const std::string& src, 
                           const std::string& msg );
      
    private:

      /// The current message level threshold
      coral::MsgLevel m_level;

      /// Use the 'new' formatted output?
      bool m_newFormat;

      /// The mutex lock
      boost::mutex m_mutex;

    };
    
  }
  
}
#endif
