#include "TestApp.h"
#include "TestEnv/Testing.h"
#include "CoralBase/Exception.h"
#include <iostream>
#include <exception>
#include <cstdlib>

int main(int argc, char *argv[])
{
  TestApp app("CS_SIMPLE_FAILOVER");
  
  TestEnvWriters& wr01 = app.addWriters().addDefault("sqlite").add("mysql").add("oracle");
  wr01.readers("sqlite").addDefault("sqlite").add("coral");  
  wr01.readers("mysql").addDefault("mysql");  
  wr01.readers("oracle").addDefault("oracle").add("coral").add("frontier");

  if(app.check(argc, argv)){ 
    //using the failover service name
    //the servicename should include two different database backends
    //the first is a none exiting one
    //the other is the failover
    app.addServiceName(0, TEXT_CORE_FAILOVER_SERVICE_0);
    
    try {
      app.runFailover(0);
    }
    catch ( coral::Exception& e )
    {
      std::cerr << "CORAL exception " << e.what() << std::endl;
      return 1;
    }
    catch ( std::exception& e )
    {
      std::cerr << "Standard C++ exception " << e.what() << std::endl;
      return 1;
    }
    catch( ... )
    {
      std::cerr << "Unknown exception ..." << std::endl;
      return 1;
    }
    std::cout << "[OVAL] Success" << std::endl;
    return 0;
  }
  return 1;
}
