#ifndef TESTCORE_TESTING_H
#define TESTCORE_TESTING_H 1

#include "TestEnv/SealSwitch.h"
#include "TestEnv/TestEnv.h"

#include "RelationalAccess/AccessMode.h"
#include "CoralCommon/Utilities.h"


namespace coral {
  class ITableDescription;
  class AttributeList;
  class ISessionProxy;
  class IConnectionService;
}

class Testing {
public:
    //constructor
    Testing(TestEnv * env);
    //destructor
    ~Testing();
    //create a session with coral::update mode
    void createSession(int index, bool reader = TESTENV_WRITER, bool sharing = true);
    //create a session with a handle
//    void createSession(int index, coral::IHandle<coral::IConnectionService> handle, coral::AccessMode mode);
    //delete all
    void deleteSession();
    //set session
    void setSession(coral::ISessionProxy * session);
    //checks with dbtype is set
    std::string getSchemaName();
//    bool isDBType(const std::string& type);
    
    //create default tables T1, T2, T3   
    void createSchemaDefault();
    //read from default tables
    void readSchemaDefault();
    //read from default tables
    void readSchemaDefault(std::string SchemaName);
    //create default view V0 based on default T1, T2, T3 tables
    //important: only use this function after calling createSchemaDefault()
    void createViewDefault();
    //fill default schema with values
    void fillSchemaDefault();
    //print default schema
    void printSchemaDefault();
    //create a simple table
    void createSimpleTable();
    
    template <class T1, class T2>
    void compareValues(const T1& source01, const T2& source02, const std::string desc) const
    {
      if(source01 != source02)
      {
        throw std::runtime_error( "Unexpected value for " + desc );
      }
    }
    
    template <class T>
    void compareValues(const long& source01, const T& source02, const std::string desc) const;


protected:
    //create a session with custom mode
    void createSessionReal(int index, coral::AccessMode mode, bool reader, bool sharing);
    //test environment
    const TestEnv& _env;
    //default tables and views    
    std::string T1;
    std::string T2;
    std::string T3;
    std::string V0;
    //session
    coral::ISessionProxy * session;

    coral::AttributeList * m_rowBuffer;
    //some tool functions
    void fillData(int i);
    void checkData(int i) const;
    void printTableInfo( const coral::ITableDescription& description ) const;
};

#endif
