#!/usr/bin/python

#   @author Alexander Kalkhof
#   @date   2008-04-16

import sys, re
if re.search("windows", sys.platform) <> None:
	platform = 1
else:
	platform = 0	

import os, popen2, select, time

import xml.sax

def getPWD():
	return os.path.realpath(os.curdir) + os.sep

def getTestPath():
	if 'CMTCONFIG' in os.environ:
		return getPWD() + os.pardir + os.sep + os.pardir + os.sep + os.pardir + os.sep + os.getenv('CMTCONFIG') + os.sep + 'tests' + os.sep + 'bin' + os.sep
	else:
		return "error"

def getCommandOutput(command):
	#split the command before, avoiding running in a shell
	#for some reasons the return tuppel is mixed as described in the documentation
	out, w, err = popen2.popen3(command)
	
	stdout = out.read()
	stderr = err.read()

	out.close()
	err.close()
	return 0, stdout, stderr

class FindTest(xml.sax.ContentHandler):
	def __init__(self, platform, tests):
		self.platform = platform
		self.tests = tests
		self.current_test = None
			
	def checkTest(self, test_name, test_type, exclude_platform, exclude_backend):
		#check if the exclude platform belongs to us
		if exclude_platform:
			#exclude exists
			#so this rule will only happen if the right platform fits
			explat = False
			#use regular expression to compare the both strings
			try:
				if re.search(exclude_platform, self.platform):
					explat = True
			except:
				print "wrong regular expression in test [%s] for [%s]" % (test_name, exclude_platform)
			
			if self.platform == exclude_platform:
				explat = True
		
			if explat:
				if exclude_backend:
					#exclude only the this backend
					self.tests.set(test_name, test_type, exclude_backend, None)
				else:
					#exclude the whole test
					#check if test exists
					self.tests.delete(test_name, test_type)
			else:
				#add no rule but at least the test
				self.tests.set(test_name, test_type, None, None)
		
		else:
			#no exclude of the platform
			#so add the "with backend" check only
			self.tests.set(test_name, test_type, exclude_backend, None)

	def startElement(self, name, attrs):
		if name == 'test':
			#get the attribute details
			test_name = attrs.get('name', None)
			test_type = attrs.get('type', None)
			timeout = attrs.get('timeout', None)
			#check if we have name and type
			if test_name and test_type:
				self.current_test = (test_name, test_type)
				self.tests.set(test_name, test_type, None, timeout)
			else:
				self.current_test = None
					
		if name == 'exclude' and self.current_test:
			#get the attribute details
			self.checkTest(self.current_test[0], self.current_test[1], attrs.get('platform', None), attrs.get('backend', None))

class FindParams(xml.sax.ContentHandler):
	def __init__(self):
		self.arguments = []
		self.optionid = 0

	def startElement(self, name, attrs):
		if name == 'option':
			self.optionid = int(attrs.get('id', 0))
		elif name == 'param':
			self.arguments.append( [ self.optionid, attrs.get('writer', None), attrs.get('reader', None) ] )

	def getResult(self):
		return self.arguments

class Test:
	def __init__(self, name, filename, testfolder, timeout):
		self.name = name
		self.filename = filename
		self.testfolder = testfolder
		self.timeout = timeout
	
	def write(self):
		self.writeQmt([])
	
	def writeQmt(self, params):
		
		testname = self.name.split('_')

		refpath = ''
		refpath += testname[1]
		for h in range(2, len(testname)):
			refpath += '_' + testname[h]
					
		refpath = os.path.realpath('../../Tests/%s/' % refpath) + '/'

		parameters01 = ''
		for h in params:
			if h[1]:
				parameters01 += ' ' + h[0] + ':' + h[1]
			else:
				parameters01 += ' ' + h[0]
			
		parameters02 = ''
		for h in params:
			if h[1]:
				parameters02 += '_' + h[0] + '-' + h[1]
			else:
				parameters02 += '_' + h[0]


		reffile = self.name + parameters02 + '.ref'
		ref = True
		#check if we have a ref file
		if not os.path.exists(refpath + reffile):
			reffile = self.name + '.ref'
			if not os.path.exists(refpath + reffile):
				ref = False
		
#		testcommand = '/usr/bin/gdb --ex r --ex bt --batch --args ' + self.filename
		testcommand = self.filename
		
		xmloutput = ''
		xmloutput += '<?xml version="1.0" ?>'
		xmloutput += '<!DOCTYPE extension  PUBLIC "-//QM/2.3/Extension//EN"  "http://www.codesourcery.com/qm/dtds/2.3/-//qm/2.3/extension//en.dtd">\n'
		xmloutput += '<extension class="LCG_QMTestExtensions.ShellCommandTest" kind="test">\n'
#		xmloutput += '<extension class="command.ExecTest" kind="test">\n'
		xmloutput += '<argument name="excluded_lines">\n'
		xmloutput += '<text />\n'
		xmloutput += '</argument>\n'
		xmloutput += '<argument name="target_group">\n'
		xmloutput += '<text>.*</text>\n'
		xmloutput += '</argument>\n'
		xmloutput += '<argument name="stderr">\n'
		xmloutput += '<text>*</text>\n'
		xmloutput += '</argument>\n'
		xmloutput += '<argument name="prerequisites">\n'
		xmloutput += '<set />\n'
		xmloutput += '</argument>'
		if ref:
			xmloutput += '<argument name="stdout">\n'
			xmloutput += '<text />\n'
			xmloutput += '</argument>'
			
			xmloutput += '<argument name="stdout_ref">\n'
			xmloutput += '<text>%s</text>\n' % reffile
			xmloutput += '</argument>\n'
			xmloutput += '<argument name="stdout_ref_path">\n'
			xmloutput += '<text>%s</text>\n' % refpath
			xmloutput += '</argument>\n'
		else:
			xmloutput += '<argument name="stdout">\n'
			xmloutput += '<text>*</text>\n'
			xmloutput += '</argument>'
		
		xmloutput += '<argument name="stdout_tol">\n'
		xmloutput += '<integer>0</integer>\n'
		xmloutput += '</argument>\n'
		xmloutput += '<argument name="exit_code">\n'
		xmloutput += '<integer>0</integer>\n'
		xmloutput += '</argument>\n'
		xmloutput += '<argument name="stdout_tag">\n'
		xmloutput += '<text>[OVAL]</text>\n'
		xmloutput += '</argument>\n'
		xmloutput += '<argument name="environment">\n'
		xmloutput += '<set />\n'
		xmloutput += '</argument>\n'		
		xmloutput += '<argument name="timeout">\n'
		xmloutput += '<integer>%s</integer>\n' % self.timeout
		xmloutput += '</argument>\n'
		xmloutput += '<argument name="description_test">\n'
		xmloutput += '<text />\n'
		xmloutput += '</argument>\n'
		xmloutput += '<argument name="command">\n'
#		xmloutput += '<text>python runTest.py %s</text>\n' % i[0]
		xmloutput += '<text>%s %s</text>\n' % (testcommand, parameters01)
		xmloutput += '</argument>\n'
		xmloutput += '<argument name="resources">\n'
		xmloutput += '<set />\n'
		xmloutput += '</argument>\n'
		xmloutput += '<argument name="stdin">\n'
		xmloutput += '<text />\n'
		xmloutput += '</argument>\n'
		xmloutput += '</extension>\n'
				
	#	print(xmloutput)

		filename = self.testfolder
		filename += testname[2]
		for h in range(3, len(testname)):
			filename += '_' + testname[h]

		filename += parameters02 + '.qmt'
		filename = filename.lower()
	#	print('[WOUT] %s' % filename)
		fd = open(filename, 'w')
		fd.write(xmloutput)
		fd.close()
				
class IntegrationTest(Test):
	def __init__(self, name, filename, excludes, testfolder, timeout):
		Test.__init__(self, name, filename, testfolder, timeout)
		#execute the test and get the stdout back
		exitcode, stdout, stderr = getCommandOutput(filename + ' -x')
		#initialize the xml sax parser
		parser02 = xml.sax.make_parser()
		#initialize the parser handler classes
		dh02 = FindParams()
		#set the content hanlder in the sax parser
		parser02.setContentHandler(dh02)
		#start parsing process
		parser02.feed(stdout)
		#transform the xml output to better format using the excludes
		realparams = self.filterExclusions(excludes, dh02.getResult())
		
		print name
		#check the result
		#if we have enough parameters
		if len(realparams) > 0 and len(realparams[0]) > 0:
			#print('[GETP] (IN) ' + name)
			#create all permutations for the test
			self.permutations = self.calcPermutations(0, self.transformParams(realparams) )
		else:
			self.permutations = None
			#print('[GETP] (--) ' + name)
			
		print self.permutations
		
	def transformParams(self, params):
		#prepare a new array
		gr = [ [] ]
		currid = 0
		#add all params splitted by the optionid
		for i in params:
			if i[0] != currid:
				gr.append([])
				currid = currid + 1
			gr[currid].append( (i[1], i[2]) )
			
		print gr
		return gr
		
	def calcPermutations(self, level, params):
		#now start the permutation process
		lenp = len(params)
		if(level == (lenp - 1)):
			result = []
			for i in params[level]:
				result.append([i])
			#print(result)
			return result
		else:
			result2 = []
			result = self.calcPermutations(level + 1, params)
			#print('[ADDM]')
			#print(params[level])
			#print('[WITH]')
			#print(result)
			for i in params[level]:
				for j in result:
					result3 = [i]
					result3.extend(j)
					result2.append(result3)
			#print('[RSLT]')
			#print(result2)
			return result2
			
	def filterExclusions(self, exclusions, result):
		res = []
		#iterate and filter all exclude backends
		for i in result:
			if(not i[1] in exclusions)and(not i[2] in exclusions):
				res.append( i )
		#return filtered array
		return res

	def write(self):
		if self.permutations:
			for i in self.permutations:
				self.writeQmt(i)

#this class handles all tests
class Tests:
	def __init__(self, test_path):
		print "look for binaries in testpath : [%s]" % test_path
		self.testpath = test_path
		self.tests = {}
		self.binaries = {}
		self.bintests = []
		self.testfolders = []
		
	def examineTests(self):
		for i in os.listdir(self.testpath):
			#split into the test name and the real binary format
			#this is important for windows and to avoid wrong files
			self.binaries[i] = self.testpath + os.sep + i
		
	def set(self, test_name, test_type, exclude_backend, timeout):
		if self.tests.has_key(test_name):
			test = self.tests[test_name]
			#check if it is the same type
			if test[1] == test_type:
				if exclude_backend:
					test[2].append( exclude_backend )
					
				if timeout:
					test[3] = timeout
		else:
			if not timeout:
				#set default timeout
				timeout = 40
			#create a new test
			self.tests[test_name] = [True, test_type, [], timeout]
			if exclude_backend:
				test[2].append( exclude_backend )
		
	def delete(self, test_name, test_type):
		if self.tests.has_key(test_name):
			#deactivate the test
			test = self.tests[test_name]
			test[0] = False
		else:
			#deactivate the test
			self.tests[test_name] = [False, test_type, []]
						
	def prepareFolders(self):
		for folder in self.testfolders:
			print "clear folder %s" % folder
			
			try:
				os.mkdir(folder)
			except:
				for i in os.listdir(folder):
					try:
						os.unlink(folder + i)
					except:
						pass
			#add the database to the folder
			try:
				os.mkdir(folder + os.sep + 'QMTest')
			except:
				pass
				#we imply the folder already exsists
			fd = open(folder + os.sep + 'QMTest' + os.sep + 'configuration', 'w')
			fd.write("<?xml version=\"1.0\" ?><!DOCTYPE extension  PUBLIC \'-//QM/2.3/Extension//EN\'  'http://www.codesourcery.com/qm/dtds/2.3/-//qm/2.3/extension//en.dtd'><extension class=\"xml_database.XMLDatabase\" kind=\"database\"/>")
			fd.close()
	
	def addFolder(self, testtype):
		testfolder = testtype + '.qms/'
		
		if not testfolder in self.testfolders:
			self.testfolders.append(testfolder)
		
		return testfolder
						
	def createTests(self):
		#get a list with all valid binary tests
		self.examineTests()
		#initial the counters
		counter_intg = 0
		counter_unit = 0
		counter_misc = 0
		#go through all keys
		for i in self.tests.keys():
			#is the test active/valid
			test = self.tests[i]
			if test[0]:
				if self.binaries.has_key(i):
					if test[1] == "integration":
						counter_intg += 1
						self.bintests.append( IntegrationTest(i, self.binaries[i], test[2], self.addFolder(test[1]), test[3]) )
					elif test[1] == "unit":
						counter_unit += 1
						self.bintests.append( Test(i, self.binaries[i], self.addFolder(test[1]), test[3] ) )
					elif test[1] == "unit_coralserver":
						counter_unit += 1
						self.bintests.append( Test(i, self.binaries[i], self.addFolder(test[1]), test[3] ) )
					else:
						counter_misc += 1
				else:
					print "WARNING: test [%s] was not found" % i
					
		print "+---- created the following tests ----"
		print "|"
		print "| [%d] integration tests" % counter_intg
		print "| [%d] unit tests" % counter_unit
		print "| [%d] other tests" % counter_misc
		print "|"
		print "+-------------------------------------"

	def writeTests(self):
		for i in self.bintests:
			i.write()

tests = Tests( os.path.realpath(getTestPath()) )
parser01 = xml.sax.make_parser()
parser01.setContentHandler( FindTest(os.getenv('CMTCONFIG'), tests) )
parser01.parse('testlist.xml')
tests.createTests()
tests.prepareFolders()
tests.writeTests()
